#include "CTerrain.h"

CTerrain::CTerrain()
{
	AktVtx=0;
	AktQuad=0;
}

CTerrain::~CTerrain()
{
	if(AktVtx)
	{
		delete vtx;
		vtx=NULL;
	}
	if(AktQuad)
	{
		delete Quads;
		Quads=NULL;
	}
}

bool CTerrain::LoadTerrain(char* filename,float size,WORD Tiles,float scale)
{
	FILE* fp;
	LPBYTE hmap = 0;
	hmap=new BYTE[size*size];
	ZeroMemory(hmap,sizeof(size*size));
	if(!(fp=fopen(filename,"rb"))) return false;
	fread(hmap,1,size*size,fp);
	fclose(fp);
	CalculateHeightfield(hmap,size,Tiles,scale);
	if(hmap)
	{
		delete hmap;
		hmap=NULL;
	}
	return true;
}

bool CTerrain::LoadTerrainBMP(char* filename,float size,float scale)
{
	FILE* fp;
	if(!(fp=fopen(filename,"rb")))	return false;
	BITMAPFILEHEADER file_header;
	BITMAPINFOHEADER bih;
	int imageIdx=0;
	BYTE tempRGB;
	fread(&file_header,sizeof(BITMAPFILEHEADER),1,fp);
	fread(&bih,sizeof(BITMAPINFOHEADER),1,fp);
	fseek(fp,file_header.bfOffBits,SEEK_SET);
	LPBYTE hmap = 0;
	hmap = new BYTE[bih.biWidth*bih.biHeight];
	ZeroMemory(hmap,bih.biWidth*bih.biHeight);
	fread(hmap,1,bih.biWidth*bih.biHeight,fp);
	fclose(fp);
	CalculateHeightfield(hmap,size,bih.biWidth,scale);
	if(hmap)
	{
		delete hmap;
		hmap=NULL;
	}
	return true;
}

void CTerrain::CalculateHeightfield(LPBYTE hmap,float size,DWORD Tiles,float scale)
{
	ScpSize = Tiles;
	Tiles--;
	AktVtx = (Tiles+1)*(Tiles+1);
	AktQuad = Tiles*Tiles*4;
	vtx = new GLVERTEX[AktVtx];
	Quads = new DWORD[AktQuad];
	GLVECTOR v1 = GLVECTOR(-size/2.0f,0,-size/2.0f);
	GLVECTOR v2 = GLVECTOR(size/2.0f,0,size/2.0f);
	DWORD countvtx = 0;
	GLVECTOR vNorm = GLVECTOR(0.0f,1.0f,0.0f);
	for(DWORD z=0;z<=Tiles;z++) for(DWORD x=0;x<=Tiles;x++)
	{
		vtx[countvtx] = GLVERTEX(GLVECTOR(v1.x+(v2.x-v1.x)/(float)Tiles*(float)x,-1.0f+255.0f/scale-(float)hmap[z*(Tiles+1)+x]/scale,v1.z+(v2.z-v1.z)/(float)Tiles*(float)z),vNorm,x,z);
		countvtx++;
	}
	countvtx = 0;
	for(z=0;z<Tiles;z++)
	{
		for(DWORD x=0;x<Tiles;x++)
		{
			Quads[countvtx++] = z*(Tiles+1)+x;
			Quads[countvtx++] = z*(Tiles+1)+x+1;
			Quads[countvtx++] = (z+1)*(Tiles+1)+x+1;
			Quads[countvtx++] = (z+1)*(Tiles+1)+x;
		}
	}
	CalcNormals();
}

void CTerrain::CalcNormals()
{
	GLVECTOR vNorm;
	DWORD count;
	for(DWORD i=0;i<AktVtx;i++)
	{
		vNorm = GLVECTOR(0,0,0);
		count = 0;
		for(DWORD b=0;b<AktQuad;b++)
		{
			if(Quads[b]==i)
			{
				DWORD x=(b/4)*4;
				vNorm += ComputeNormal(
					GLVECTOR(vtx[Quads[x]].x,vtx[Quads[x]].y,vtx[Quads[x]].z),
					GLVECTOR(vtx[Quads[x+1]].x,vtx[Quads[x+1]].y,vtx[Quads[x+1]].z),
					GLVECTOR(vtx[Quads[x+2]].x,vtx[Quads[x+2]].y,vtx[Quads[x+2]].z));
				count++;
			}
		}
		vtx[i].nx = vNorm.x/(float)count;
		vtx[i].ny = vNorm.y/(float)count;
		vtx[i].nz = vNorm.z/(float)count;
	}
}

void CTerrain::render()
{
//	glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	float g[4] = {128.0f/255.0f,96.0f/255.0f,0.0f/255.0f,1.0f};
	glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,g);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,texsand);
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glVertexPointer(3,GL_FLOAT,sizeof(GLVERTEX),&vtx[0].x);
	glNormalPointer(GL_FLOAT,sizeof(GLVERTEX),&vtx[0].nx);
	glTexCoordPointer(2,GL_FLOAT,sizeof(GLVERTEX),&vtx[0].tu);
	glDrawElements(GL_QUADS,AktQuad,GL_UNSIGNED_INT,Quads);
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	glDisable(GL_TEXTURE_2D);
}