// GLSys.h: Schnittstelle fr die Klasse GLSys.
//
//////////////////////////////////////////////////////////////////////

#define WIN32_LEAN_AND_MEAN
#define WGL_WGLEXT_PROTOTYPES

#define PI 3.141592654f

#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <gl\glaux.h>
#include <stdio.h>
#include <math.h>

//#include "glext.h"
//#include "wglext.h"
//#include "glxext.h"

#ifndef GLSYSMATH

class GLMATRIX
{
public:
	float m[4][4];
};

class GLVECTOR
{
public:
	float x;
	float y;
	float z;
	GLVECTOR(float a_x,float a_y,float a_z);
	void operator+=(GLVECTOR);
	void operator-=(GLVECTOR);
	bool operator==(GLVECTOR);
	GLVECTOR operator+(GLVECTOR add);
	GLVECTOR operator-(GLVECTOR sub);
	GLVECTOR operator*(float scale);
	GLVECTOR operator/(float scale);
	GLVECTOR();
};

class GLVERTEX
{
public:
	float x;
	float y;
	float z;
	float nx;
	float ny;
	float nz;
	float tu;
	float tv;
	GLVERTEX(GLVECTOR a_point,GLVECTOR a_normal);
	GLVERTEX(GLVECTOR a_point);
	GLVERTEX(GLVECTOR a_point,GLVECTOR a_normal,float a_tu,float a_tv);
	GLVERTEX();
};

struct CAMERA
{
	GLVECTOR  vPos;
	float	  XDeg;
	float	  YDeg;
	float	  FOV;
	float	  cNear;
	float	  cFar;
};

class GLSys
{
public:
//	HPBUFFERARB				hBuf;
	HGLRC					hRC;
	HDC						hDC;
	HRESULT InitOpenGLWindow(HWND hWnd);
	HRESULT InitOpenGLTexture(DWORD Width,DWORD Height);
	HRESULT InitOpenGLFullscreen(HINSTANCE hInstance,HWND *hWnd,WNDCLASS *wc,char *classname,char *title,DWORD Width,DWORD Height,DWORD BitDepth);
	HRESULT SetProjection(float Format,float fFOV,float fNearPlane,float fFarPlane);
	HRESULT ShutDown(void);
	GLVECTOR GetPointOnY(CAMERA ViewDat,int wndx,int wndy,int x,int y,float pointy);
	GLVECTOR GetPointOnXZ(CAMERA ViewDat,int wndx,int wndy,int y,float pointx,float pointz);
	GLVECTOR GetSourcePointOnY(CAMERA ViewDat,int wndx,int wndy,int x,int y,float pointy);
	GLVECTOR GetPointOnCamera(CAMERA ViewDat,int wndx,int wndy,int x,int y);
	void CreateEmptyTexture(GLuint *texture,BYTE colors,GLenum format,bool linear,DWORD Width,DWORD Height);
	void CreateTexture(GLuint *texture,char *strName,BYTE colors,GLenum format,bool linear,bool mipmap);
	bool CreateTextureAlpha(GLuint *texture,char *strName,char *strNameAlpha,bool linear,bool mipmap);
	bool LoadTGA(GLuint *tex,char* filename);
	void Screenshot(char *name);

	GLSys();
	virtual ~GLSys();
};

BOOL TestIntersection(GLVECTOR orig,GLVECTOR dir,GLVECTOR vert0,GLVECTOR vert1,GLVECTOR vert2);
BOOL IntersectTriangle(GLVECTOR orig,GLVECTOR dir,GLVECTOR vert0,GLVECTOR vert1,GLVECTOR vert2,FLOAT *t,FLOAT *u,FLOAT *v);
GLVERTEX VertexMatrixMultiply(GLVERTEX &src,GLMATRIX mat);
GLVECTOR VectorMatrixMultiply(GLVECTOR &src,GLMATRIX mat);
HRESULT MatrixInvert(GLMATRIX& q,GLMATRIX& a);
GLMATRIX CreateRotateMatrix(float x,float y,float z);
GLVECTOR Round(GLVECTOR vSrc,float base);
void SetTranslateMatrix(GLMATRIX& mat,float tx,float ty,float tz);
void SetScaleMatrix(GLMATRIX& mat,float sx,float sy,float sz);
void SetRotateXMatrix(GLMATRIX& mat,float d_angle);
void SetRotateYMatrix(GLMATRIX& mat,float d_angle);
void SetRotateZMatrix(GLMATRIX& mat,float d_angle);
void ClearMatrix(GLMATRIX& mat);
void MatrixMultiply(GLMATRIX& q,GLMATRIX& a,GLMATRIX& b);
void SetIdentityMatrix(GLMATRIX& mat);
GLVECTOR Normalize(GLVECTOR vec);
GLVECTOR CrossProduct(GLVECTOR v1,GLVECTOR v2);
float DotProduct(GLVECTOR v1,GLVECTOR v2);
double AngleBetweenVectors(GLVECTOR v1,GLVECTOR v2);
float Magnitude(GLVECTOR v1);
GLVECTOR ComputeNormal(GLVECTOR a,GLVECTOR b,GLVECTOR c);

#endif
#define GLSYSMATH